@@ -1,3 +1,10 @@
+import urllib
+try:
+    import ujson as json
+except ImportError:
+    import json
+
+
 class HTTPError(Exception):
     def __init__(self, code, message):
         self.code = code
@@ -17,9 +24,9 @@ import socket
 
 
 class Connection(object):
-    def __init__(self, port=DEFAULT_PORT, host=DEFAULT_HOST, retry=1):
+    def __init__(self, port=DEFAULT_PORT, host=DEFAULT_HOST, reconnect=1):
         self.url = "http://%s:%s/%%s" % (host, port)
-        self.retry = retry
+        self.reconnect = reconnect
         self.conn = httplib.HTTPConnection(host, port, timeout=None)
 
     def reconnect(self):
@@ -28,7 +35,7 @@ class Connection(object):
 
     def request(self, verb, url, body=None):
         attempt = 0
-        while attempt < self.retry:
+        while attempt < self.reconnect:
             attempt += 1
             try:
                 self.conn.request(verb, url, body)
@@ -83,7 +90,8 @@ else:
 
     class TornadoAsyncClient(object):
         def __init__(self, port=DEFAULT_PORT, host=DEFAULT_HOST):
-            self.url = "http://%s:%s/%%s" % (host, port)
+            self.root_url = "http://%s:%s" % (host, port)
+            self.url = "%s/%%s" % self.root_url
             self.client = AsyncHttpClient()
             self._get = self.get
             self._put = self.put
@@ -101,6 +109,27 @@ else:
         def put(self, key, value, callback=None):
             self.client.fetch(self.url % key, method="PUT", body=value, callback=callback)
 
+        def get_multiple(self, keys, callback=None):
+            def _callback(response):
+                callback(json.loads(response.body))
+
+            self.client.fetch(self.url % ("?%s" % urllib.urlencode({"key": keys}, True)),
+                              method="GET", body=json.dumps(keys), callback=_callback)
+
+        def get_multiple_mapped(self, keys, callback):
+            def _callback(values):
+                mapping = {}
+                i = iter(values)
+                for key in keys:
+                    value = i.next()
+                    mapping[key] = value
+                callback(mapping)
+
+            self.get_multiple(keys, _callback)
+
+        def put_multiple(self, mapping, callback=None):
+            self.client.fetch(self.root_url, method="PUT", body=json.dumps(mapping), callback=callback)
+
 # endregion
 
 #region Redis
@@ -161,10 +190,6 @@ try:
 except ImportError, ex:
     redis = False
 
-    class RedisCacheClient():
-        def __init__(self, *args, **kwargs):
-            raise ex
-
 #endregion
 
 __all__ = ["Client", "Connection", "TornadoAsyncClient", "RedisCacheClient", "RedisAsyncCacheClient"]
\ No newline at end of file
